package java.io;

import sun.reflect.CallerSensitive;
import sun.reflect.misc.ReflectUtil;
import sun.reflect.Reflection;
import java.lang.reflect.Field;

public class ObjectStreamField implements Comparable<Object>
{
    private final String name;
    private final String signature;
    private final Class<?> type;
    private final boolean unshared;
    private final Field field;
    private int offset;
    
    public ObjectStreamField(String s, Class<?> clazz) {
        this(s, clazz, false);
    }
    
    public ObjectStreamField(String name, Class<?> type, boolean unshared) {
        this.offset = 0;
        if (name == null) {
            throw new NullPointerException();
        }
        this.name = name;
        this.type = type;
        this.unshared = unshared;
        this.signature = getClassSignature(type).intern();
        this.field = null;
        return;
    }
    
    ObjectStreamField(String name, String s, boolean unshared) {
        this.offset = 0;
        if (name == null) {
            throw new NullPointerException();
        }
        this.name = name;
        this.signature = s.intern();
        this.unshared = unshared;
        this.field = null;
        switch (s.charAt(0)) {
            case 'Z': {
                this.type = Boolean.TYPE;
                break;
            }
            case 'B': {
                this.type = Byte.TYPE;
                break;
            }
            case 'C': {
                this.type = Character.TYPE;
                break;
            }
            case 'S': {
                this.type = Short.TYPE;
                break;
            }
            case 'I': {
                this.type = Integer.TYPE;
                break;
            }
            case 'J': {
                this.type = Long.TYPE;
                break;
            }
            case 'F': {
                this.type = Float.TYPE;
                break;
            }
            case 'D': {
                this.type = Double.TYPE;
                break;
            }
            case 'L':
            case '[': {
                this.type = Object.class;
                break;
            }
            default: {
                throw new IllegalArgumentException("illegal signature");
            }
        }
        return;
    }
    
    ObjectStreamField(Field field, boolean unshared, boolean b) {
        this.offset = 0;
        this.field = field;
        this.unshared = unshared;
        this.name = field.getName();
        Class<Object> type = field.getType();
        this.type = ((b || type.isPrimitive()) ? type : Object.class);
        this.signature = getClassSignature(type).intern();
    }
    
    public String getName() {
        return this.name;
    }
    
    @CallerSensitive
    public Class<?> getType() {
        if (System.getSecurityManager() != null && ReflectUtil.needsPackageAccessCheck(Reflection.getCallerClass().getClassLoader(), this.type.getClassLoader())) {
            ReflectUtil.checkPackageAccess(this.type);
        }
        return this.type;
    }
    
    public char getTypeCode() {
        return this.signature.charAt(0);
    }
    
    public String getTypeString() {
        return this.isPrimitive() ? null : this.signature;
    }
    
    public int getOffset() {
        return this.offset;
    }
    
    protected void setOffset(int offset) {
        this.offset = offset;
    }
    
    public boolean isPrimitive() {
        int char1 = this.signature.charAt(0);
        return char1 != 'L' && char1 != '[';
    }
    
    public boolean isUnshared() {
        return this.unshared;
    }
    
    @Override
    public int compareTo(Object o) {
        ObjectStreamField objectStreamField = (ObjectStreamField)o;
        int primitive = this.isPrimitive();
        if (primitive != (objectStreamField.isPrimitive() ? 1 : 0)) {
            return primitive ? -1 : 1;
        }
        return this.name.compareTo(objectStreamField.name);
    }
    
    @Override
    public String toString() {
        return this.signature + 32 + this.name;
    }
    
    Field getField() {
        return this.field;
    }
    
    String getSignature() {
        return this.signature;
    }
    
    private static String getClassSignature(Class<?> componentType) {
        StringBuilder sb = new StringBuilder();
        while (componentType.isArray()) {
            sb.append('[');
            componentType = componentType.getComponentType();
        }
        if (componentType.isPrimitive()) {
            if (componentType == Integer.TYPE) {
                sb.append('I');
            }
            else if (componentType == Byte.TYPE) {
                sb.append('B');
            }
            else if (componentType == Long.TYPE) {
                sb.append('J');
            }
            else if (componentType == Float.TYPE) {
                sb.append('F');
            }
            else if (componentType == Double.TYPE) {
                sb.append('D');
            }
            else if (componentType == Short.TYPE) {
                sb.append('S');
            }
            else if (componentType == Character.TYPE) {
                sb.append('C');
            }
            else if (componentType == Boolean.TYPE) {
                sb.append('Z');
            }
            else if (componentType == Void.TYPE) {
                sb.append('V');
            }
            else {
                throw new InternalError();
            }
        }
        else {
            sb.append(76 + componentType.getName().replace('.', '/') + 59);
        }
        return sb.toString();
    }
}
